use diesel::pg::Pg;
use diesel_async::AsyncConnection;
use diesel_async::async_connection_wrapper::AsyncConnectionWrapper;
use diesel_migrations::{EmbeddedMigrations, MigrationHarness};
use tokio::task::JoinError;

/// Run Diesel migrations using an async connection.
/// Only works with Postgres.
pub async fn run_migrations<A>(
    async_connection: A,
    migrations: EmbeddedMigrations,
) -> Result<(), JoinError>
where
    A: AsyncConnection<Backend = Pg> + 'static,
{
    let mut async_wrapper = AsyncConnectionWrapper::<A>::from(async_connection);

    tokio::task::spawn_blocking(move || {
        async_wrapper.run_pending_migrations(migrations).unwrap();
    })
    .await
}
