use crate::diesel::DieselError;
use crate::diesel::get_connection::{GetConnection, GetConnectionError};
use crate::diesel::pool::PgPool;
use deadpool_diesel::Status;
use deadpool_diesel::postgres::BuildError;
use derive_more::From;
use diesel_async::pooled_connection::deadpool::Object;
use diesel_async::{AsyncConnection, AsyncPgConnection};
use lib::diesel::pool::create_pool_from_url_with_size;

#[derive(Clone)]
pub struct PoolStub(PgPool);

#[derive(Debug, PartialEq, From)]
pub enum Error {
    Connection(diesel::ConnectionError),
    Database(DieselError),
}

pub async fn setup_test_transaction(url: impl AsRef<str>) -> Result<AsyncPgConnection, Error> {
    let mut conn = AsyncPgConnection::establish(url.as_ref()).await?;
    conn.begin_test_transaction().await?;
    Ok(conn)
}

pub async fn create_test_pool_url_with_size(
    url: impl Into<String>,
    size: usize,
) -> Result<PoolStub, BuildError> {
    let pool = create_pool_from_url_with_size(url, size)?;
    Ok(PoolStub(pool))
}

impl GetConnection for PoolStub {
    async fn get(&self) -> Result<Object<AsyncPgConnection>, GetConnectionError> {
        let mut conn = self.0.get().await?;
        conn.begin_test_transaction().await?;
        Ok(conn)
    }
    fn status(&self) -> Status {
        unimplemented!("PoolStub does not support status")
    }
}
